/*
 * dimmer.c
 *
 * Created: 2012-12-31 00:15:01
 *  Author: user
 */ 

#define F_CPU 8000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <inttypes.h>
#include "hardware.h"
#include "dimmer.h"
#include "sensor.h"

#define DIMMER_PERIOD		128
#define DIMMER_DELAY		128
#define TICK_PERIOD			16

struct	{
	volatile char tick;
	char stage;
	char level;
	uint16_t time;
	uint16_t rising;
	uint16_t duration;
	char delay;
	uint16_t falling;
}dimm;

//--------------------------------------------------
void DimmInit(void)
{	
    TCCR0A = (1<<COM0B1 | 1<<COM0B0 | 1<<WGM00 | 1<<WGM01); 		//fast PWM
	TCCR0B = (1<<WGM02 | 1<<CS01);
	OCR0A = DIMMER_PERIOD;
	OCR0B = DIMMER_PERIOD;
	TIMSK0 = (1<<TOIE0);	
	//dimm.rising = 10;			//10 - 3s
	//dimm.duration = 40;			//40 ~ 10s
	//dimm.falling = 30;			//30 - 8s
}

//--------------------------------------------------
void DimmerAction(void)
{
	if (CF1_GET == 0)					//no jumper - refresh mode - always rising
	{								
		dimm.stage = DIMMER_RISE;	
	}
	else							//insert jumper
	{								
		if (dimm.stage == DIMMER_FALL)				//if considerable falling then rising
		{
			if (dimm.level <= (DIMMER_PERIOD - 20))
				dimm.stage = DIMMER_RISE;				
		}
		if (dimm.stage == DIMMER_BRIGHT)			//if bright then falling
			dimm.stage = DIMMER_FALL;
		if (dimm.stage == DIMMER_IDLE)				//if idle then rising
			dimm.stage = DIMMER_RISE;
	}
}

//--------------------------------------------------
void DimmProcess(void)
{	
	//if (dimm.next)
	if (dimm.tick > TICK_PERIOD) 
	{
		//dimm.next = 0;
		dimm.tick -= TICK_PERIOD;
	
		//----------------------------------------------
		if (dimm.stage == DIMMER_IDLE) 
		{
			dimm.level = 0;
			dimm.time = 0;
		}	
	
		//----------------------------------------------
		else if (dimm.stage == DIMMER_RISE)
		{
			if (dimm.time >= dimm.rising)
			{
				dimm.time = 0;
				if (dimm.level < DIMMER_PERIOD) dimm.level++;
				else 
				{
					dimm.stage = DIMMER_BRIGHT;
					dimm.delay = DIMMER_DELAY;
				}									
			}
			dimm.time++;
		}
	
		//----------------------------------------------
		else if (dimm.stage == DIMMER_BRIGHT)
		{
			if (dimm.time >= dimm.duration)
			{
				dimm.time = 0;			
				dimm.level = DIMMER_PERIOD;
				if (dimm.delay) dimm.delay--;
				else dimm.stage = DIMMER_FALL;
			}
			dimm.time++;
		} 
	
		//----------------------------------------------
		else if (dimm.stage == DIMMER_FALL)
		{
			if (dimm.time >= dimm.falling)
			{
				dimm.time = 0;
				if (dimm.level > 0) dimm.level--;
				else dimm.stage = DIMMER_IDLE;
			}		
			dimm.time++;
		}

		//----------------------------------------------
		else  dimm.stage = DIMMER_FALL;
				
		OCR0B = (DIMMER_PERIOD - dimm.level);
	}
}

//--------------------------------------------------
ISR(TIM0_OVF_vect)
{
	dimm.tick++;
}