/*
 * lighter.c
 *
 * Created: 2012-12-21 13:34:18
 *  Author: dsosnowski
 */ 

#define F_CPU 8000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "hardware.h"
#include "sensor.h"
#include "dimmer.h"
#include "wdog.h"

#define TOUCH_PERIOD			10
#define OFFSET_PERIOD			200
#define LED_PULSE				100
#define INPUT_TRIG				10
#define INPUT_ON				1
#define INPUT_OFF				0

void HardInit(void);
void SettingsUpdate(void);
void InputScan(void);

extern struct {
	volatile char process;
	uint16_t falling;
	uint16_t average;
	char delta;
	char occupied;
	char release;
	uint16_t mem[SENSOR_MEM];
	char index;
	char action;
}capacitive;

struct {
	volatile char tick;
	char stage;
	char level;
	uint16_t time;
	uint16_t rising;
	uint16_t duration;
	char delay;
	uint16_t falling;
}dimm;

struct{
	char touch;
	char offset;
	char led;
}rules;

struct  {
	char channel;
	char res1;
	char res2;
	char res3;
	char low;
	char high;
	char action;
} sett;


//---------------------------------------------------------
int main(void)
{
	HardInit();
	
	//--------------------------
	rules.offset = 0;
	while (rules.offset < SENSOR_MEM)
	{
		CapOffset();
		SettingsUpdate();
		rules.offset++;
	}
	//capacitive.delta = 0xFF;
	sett.low = 0xFF;
	sett.high = 0xFF;
	capacitive.occupied = 0xFF;	
	
	WdtSet(7);

	//-----------------------------------------------------
    while(1)
    {
		//------------------------------------------------
		if (rules.touch < TOUCH_PERIOD) rules.touch++;
		else
		{
			rules.touch = 0;
			//------------------------------------------------
			CapProcess();
			CapResult(); 
			if (capacitive.action == CAP_PRESSED)
			{
				capacitive.action = 0;
				DimmerAction();
			}					
			//------------------------------------------------
			InputScan();
			if (sett.action == INPUT_ON)
			{
				sett.action = INPUT_OFF;
				DimmerAction();
			}
		}
		
		//------------------------------------------------
		if (rules.offset < OFFSET_PERIOD) rules.offset++;
		else rules.offset = 0;
		//--------------------------
		if (rules.offset == 0)
		{	
			CapOffset();
			if (capacitive.index == 0) rules.led = LED_PULSE;			
		}
		//--------------------------
		if (rules.offset == 1)
		{
			if (capacitive.index == 0) SettingsUpdate();
		}		
					 
		//------------------------------------------------
		DimmProcess();			//relative of timer
		if (dimm.stage == DIMMER_BRIGHT || dimm.stage == DIMMER_RISE)
		{
			rules.led = LED_PULSE;			
			EXT_ON;
		}
		else		
			EXT_OFF;			
		
		//------------------------------------------------
		if (rules.led)
		{
			LED_ON;
			rules.led--;
		} 		
		else LED_OFF;

		//------------------------------------------------
		_delay_ms(1);
		WdtReset();		
    }
}
//--------------------------------------------------
void InputScan(void)
{	
	if (INPUT_GET == 0) 
	{
		if (sett.high <= INPUT_TRIG) sett.high++;		
		if (sett.high == INPUT_TRIG)
		{
			if (CF2_GET) sett.action = INPUT_ON;
		}			
		sett.low = 0;
	}
	else
	{
		if (sett.low <= INPUT_TRIG) sett.low++;		
		if (sett.low == INPUT_TRIG)
		{
			if (CF2_GET == 0) sett.action = INPUT_ON;
		}			
		sett.high = 0;		
	}
}	

//--------------------------------------------------
void SettingsUpdate(void)
{	
	sett.channel++;	
	if (sett.channel == 0) sett.channel = 1;	
	if (sett.channel > 3) sett.channel = 1;
	while (ADCSRA & (1<<ADSC));
	ADMUX = (sett.channel);
	ADCSRA = (1<<ADEN | 1<<ADSC | 1<<ADPS1 | 1<<ADPS2);
	while (ADCSRA & (1<<ADSC));
	
	if		(sett.channel == 1) 
	{	
		sett.res1 = (0xFF - ADCH);
		if (sett.res1 == 0) sett.res1 = 1;
		dimm.falling = (sett.res1 << 0);		
		dimm.rising = 3;
	}	
	else if (sett.channel == 2) 
	{	
		sett.res2 = (0xFF - ADCH);
		if (sett.res2 == 0) sett.res2 = 1;
		dimm.duration = (sett.res2 << 4);
	}	
	else if (sett.channel == 3) 
	{	
		sett.res3 = (0xFF - ADCH);
		sett.res3 >>= 3;
		if (sett.res3 < 2) sett.res3 = 2;
		capacitive.delta = sett.res3;
		//dimm.rising = (sett.res3 >> 3);		
	}
}

//---------------------------------------
void HardInit(void)
{
	INPUT_PORT = 0x00;
	DDRB = (1<<INPUT_EXT);

	IO_PORT = (1<<IO_CAP | 1<<IO_CF1 | 1<<IO_CF2);
	DDRA = (1<<IO_DRV | 1<<IO_LED);
	ACSR = (1<<ADC);		//turn off comparator
	PRR = (1<<PRUSI);		//turn off USI

	ADMUX = (1);					//ref = Vcc, channel = 1
	ADCSRA = (1<<ADEN | 1<<ADPS1);	//presc 64
	ADCSRB = (1<<ADLAR);		
	DIDR0 = (1<<ADC1D | 1<<ADC2D | 1<<ADC3D);
	ADCSRA = (1<<ADEN | 1<<ADSC | 1<<ADPS1);		//init conv
	
	SensorInit();
	DimmInit();
	sei();

}

