/*
 * sensor.c
 *
 * Created: 2012-12-26 23:54:53
 *  Author: user
 */ 


#define F_CPU 8000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <inttypes.h>
#include "hardware.h"
#include "sensor.h"

#define CORE_CLOCK		8000000
#define SENSOR_PERIOD	4096
/*
#define INPUT_PORT		PORTB
#define INPUT_PIN		2

#define IO_PORT			PORTA
#define IO_CAP			0		//PCINT0, ADC0
#define IO_DRV			7		//H activ
*/
#define DISABLE_CAP		GIMSK &= ~(1<<PCIE0)
#define CLEAR_CAP	 	GIFR |= (1<<PCIF0)
#define ENABLE_CAP		GIMSK |= (1<<PCIE0)
#define CAPPIN_OFF		(DDRA &= ~(1<<IO_CAP))
#define CAPPIN_ON		(DDRA |= (1<<IO_CAP))
#define CAPPIN_TOL		(IO_PORT &= ~(1<<IO_CAP))
#define CAPPIN_TOH		(IO_PORT |= (1<<IO_CAP))
#define CAPPIN_GET		(PINA & (1<<IO_CAP))

#define TIMER_START		TCCR1B = (1<<WGM13 | 1<<WGM12 | 1<<CS10)		//fast PWM, TOP on ICR, OVF on TOP
#define TIMER_STOP		TCCR1B = (1<<WGM13 | 1<<WGM12)
#define TIMER_RESET		TCNT1 = 0
#define TIMER_VAL		TCNT1

#define CPROCESS_IDLE	0
#define CPROCESS_START	1
#define CPROCESS_READY	2
#define CPROCESS_ERR	3

//#define SENSOR_MEM	12
#define SENSOR_HOLE		4
//#define CAP_DELTA		30
#define OCCUPIED_CYCLE	14

struct {
	volatile char process;
	uint16_t falling;
	uint16_t average;
	char delta;
	char occupied;
	char release;
	uint16_t mem[SENSOR_MEM];
	char index;
	char action;
}capacitive;

//----------------------------------------------
void SensorInit(void)
{
	TCCR1A = (1<<WGM11);						//fast PWM, TOP on ICR, OVF on TOP
	TCCR1B = (1<<WGM13 | 1<<WGM12);
	ICR1 = SENSOR_PERIOD;
	TIMSK1 = (1<<TOIE1);
	PCMSK0 |= (1<<IO_CAP);
}

//----------------------------------------------
void CapOffset(void)
{
	char temp;
	char i;
	//-------------------------------------------
	capacitive.average = 0;
	i = capacitive.index;
	temp = 0;
	//-------------------------------------------
	while (temp < (SENSOR_MEM - SENSOR_HOLE))			
	{
		i++;
		if (i >= SENSOR_MEM) i = 0;
		capacitive.average += capacitive.mem[i];
		temp++;				
	}
	capacitive.average /= (SENSOR_MEM - SENSOR_HOLE);

	//-------------------------------------------
	capacitive.index++;
	if (capacitive.index >= SENSOR_MEM) capacitive.index = 0;
	capacitive.mem[capacitive.index] = capacitive.falling;	
}

//----------------------------------------------
void CapProcess(void)
{
	DISABLE_CAP;
	CAPPIN_TOH;			//pin to H
	CAPPIN_ON;
	
	capacitive.process = CPROCESS_START;
	TIMER_STOP;
	TIMER_RESET;
	TIMER_START;	
	CLEAR_CAP;
	ENABLE_CAP;
	CAPPIN_OFF;
	CAPPIN_TOL;
	
	//--------------------------------
	while (capacitive.process == CPROCESS_START);
	
	DISABLE_CAP;
	TIMER_STOP;
	CAPPIN_TOH;			//pin to H
	CAPPIN_ON;
	
	if (capacitive.process == CPROCESS_READY) 
		capacitive.falling = TIMER_VAL;
	else capacitive.falling = 0;
}

//----------------------------------------------
char CapResult(void)
{
	//--------------------------------------------
	//get cycles of rising
	if (capacitive.falling > (capacitive.average + capacitive.delta))
	{
		if (capacitive.occupied <= OCCUPIED_CYCLE) capacitive.occupied++;
		if (capacitive.occupied == OCCUPIED_CYCLE) capacitive.action = CAP_PRESSED;
		capacitive.release = 0;
	}
		
	//--------------------------------------------
	//cycles of falling
	//if (capacitive.falling < (capacitive.average + CAP_DELTA_OFF))
	else
	{
		if (capacitive.release <= OCCUPIED_CYCLE) capacitive.release++;
		else capacitive.occupied = 0;
	}			 
	
	//--------------------------------------------
	//if (capacitive.cycle == 0)
	//	capacitive.action = CAP_FREE; 
	return capacitive.action;
}

//-----------------------------------------------
ISR(TIM1_OVF_vect)
{
	TIMER_STOP;
	capacitive.process = CPROCESS_ERR;
}

//---------------------------------------
ISR(PCINT0_vect)
{
	TIMER_STOP;
	capacitive.process = CPROCESS_READY;
}